#include <iostream> 
#include "stdio.h"
#include "math.h"
#include "TROOT.h"
#include "TSystem.h"
#include "TClass.h"
#include "TClassTable.h"
#include "TApplication.h"
#include "TGApplication.h"
#include "TDatime.h"
#include "TGClient.h"
#include "TGWindow.h"
#include "TObject.h"
#include "TRint.h"
#include "TString.h"
#include "SPMRoot.h"
#include "log.hh"
#include "version.h"

using namespace std;

extern void InitGui();
VoidFuncPtr_t initfuncs[] = {InitGui,0};
TROOT root("SPM","SPM",initfuncs);

void banner(char* controler)
{
  log(10," ");
  log(10,"      *******************************************");
  log(10,"      **                                       **");
  log(10,"      **          S P M R o o t  v %2d.%02d       **",SPMROOT_VERSION,SPMROOT_REVISION);
  log(10,"      **                                       **");
  log(10,"      ** a ROOT based data acquisition system  **");
  log(10,"      **                                       **");
  log(10,"      **     (c) 2003-%4d A. A. P. Suaide     **",PELLIB_YEAR);  
  log(10,"      **                                       **");
  log(10,"      *******************************************");
  log(10," ");
  log(10,"Running SPMRoot. type spmroot -help for options ");
  log(10," ");
  log("SPMRoot has started");
  TString A = "CAMAC crate controler: ";
  A+=controler;
  log(A.Data());
}
int main(int argc, char** argv)
{   
  int w = 450;
  int h = 570;
  bool debug = false;
  bool cint = false;
  bool help = false;
  TString device = "/dev/c111a0";
  if(argc>1) for(int i=1;i<argc;i++) 
  {
    bool id = false;
		TString a = argv[i];
    if(!strcmp(argv[i],"-debug")) { debug = true; id = true;}
    if(!strcmp(argv[i],"-cint"))  { cint = true; id = true;}
    if(!strcmp(argv[i],"-help"))  { help = true; id = true;}
    if(!strcmp(argv[i],"-d"))     { debug = true; id = true;}
    if(!strcmp(argv[i],"-c"))     { cint = true; id = true;}
    if(!strcmp(argv[i],"-h"))     { help = true; id = true;}
    if(a.BeginsWith("/dev"))      { device = a; id = true;}
    if(a.EndsWith(".fil"))        { device = a; id = true;}
    if(!id)
    {
      cout <<endl<<"SPMRoot::error()"<<endl;
      cout <<"Option "<<argv[i]<<" is not valid! Use spmroot -h for help"<<endl;
      return 0;
    }
  }
  if(help)
  {
    char line[100];
    sprintf(line,"SPMRoot version %2d.%02d (c) 2003-%4d by A. A. P. Suaide",SPMROOT_VERSION,SPMROOT_REVISION,PELLIB_YEAR);
    cout <<line<<endl;
    cout <<"usage: spmroot [-h|help] [-d|debug] [-c|cint] [/dev/...]"<<endl;
    cout <<"-h or -help    displays this message"<<endl;
    cout <<"-d or -debug   turns on debug mode and display event by event information"<<endl;
    cout <<"-c or -cint    enables ROOTCint interpreter"<<endl;
    cout <<"/dev/...       sets the CAMAC device (default = /dev/c111a0)"<<endl;
    return 0;
  }
    
  char logname[100];
  TDatime now;
  sprintf(logname,"SPMRoot.%8d.log",now.GetDate());
  appendLog(logname);
  gStyle->SetPalette(1,0);
  gStyle->SetOptLogz(1);  

  if(!cint)
  {
    TApplication r("SPMRoot",&argc,argv);
    SPMRoot SPM(gClient->GetRoot(),w,h,debug);
    SPM.setCamac((char*)device.Data());
    banner((char*)device.Data());
    r.Run();
  }
  else
  {
    TRint r((const char*)"RU",&argc,argv,0,0,kTRUE);
    r.SetPrompt("SPMRoot > ");
    SPMRoot SPM(gClient->GetRoot(),w,h,debug);
    SPM.setCamac((char*)device.Data());
    banner((char*)device.Data());
    r.Run();
  }
  closeLog();
  return 1;
}
