#include "adac+.h"
#include "TROOT.h"
TString baseName;

int INITLINE;

/////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
  if(argc==1)
  {
    cout <<"usage: adac+ filename\n";
    return 0;
  }
  baseName = argv[1];
  if(baseName.EndsWith(".evs")) baseName = baseName(0,baseName.Length()-4);
  
	// read file
	TPrg* RAW = readFile((char*)baseName.Data());	
	RAW->print("RAW FILE");
	
	//expand program
	TPrg* EXP = expand(RAW);
	if(EXP) EXP->print("EXPANDED FILE");
	
	//Compact program
	TPrg* COMPACT = compact(EXP);
	if(COMPACT) COMPACT->print("COMPACTED FILE");
	
	//Compile it!
	compile(COMPACT);
	if(COMPACT) COMPACT->print("COMPACTED FILE - Compiled");
	
  cout <<"NO ERRORS !!\n";
}
/////////////////////////////////////////////////////////////////
void err(int C, TPrgLine* P)
{
  TString M;
  if(C==IOERR)             M = "I/O error";
  if(C==SYNTAXERR)         M = "Syntax Error";
  if(C==OPERERR)           M = "Operator error";
  if(C==LOOPERR)           M = "Loop error";
  if(C==LOOPNOTSTARTED)    M = "ENDLOOP without LOOP";
  if(C==NESTEDLOOPS)       M = "Nested loops";
  if(C==LOOPSIZEERR)       M = "Loop size error";
  if(C==LOOPLABELERR)      M = "Can not have jump labels inside loops";
  if(C==LOOPNOTCLOSED)     M = "Loop not closed";
  if(C==VARIABLENOTDEF)    M = "Assembly time variable not defined";
  if(C==NOTVALIDCOMMAND)   M = "Syntax Error: Command invalid";
  if(C==INVALIDNUMBERCMD)  M = "Syntax Error: Wrong number of parameters";
  if(C==BITPATTERR)        M = "Bit Pattern error";
  if(C==INITLOOPERR)       M = "INIT inside a loop";
  if(C==INITMANYERR)       M = "More than one INIT statement";
  if(C==INITNONEERR)       M = "Missing INIT statement in program";
	
	TPrgLine* LINE = P;
	while(P->getParent()) P = P->getParent();
	
	cout <<"ERROR! "<<M.Data()<<" in line "<<P->getLineNumber()<<endl;
	cout <<"   ==> "<<P->getCodeRaw()<<endl;
	
  exit(0);
}
/////////////////////////////////////////////////////////////////
TPrg* readFile(char* fileName)
{
  TString file=fileName;
  file+=".evs";
  FILE *in = fopen((char*)file.Data(),"r");
  if(in==0) err(IOERR);
	
	TPrg* RAW = new TPrg();
	
  char line[400];
  while(!feof(in))
  {
    fgets(line,400,in);
    TPrgLine* LINE = RAW->addLine(line);
  }
  fclose(in);
  return RAW;
}
/////////////////////////////////////////////////////////////////
TPrg* expand(TPrg* RAW)
{
  if(!RAW) return NULL;
	if(RAW->getNLines()==0) return NULL;
	
	TPrg* EXP = new TPrg();
	
  int nL = 0;
  int nI = 0;
  int loop1 = 0;
  int loop2 = 0;
  
	for(int i=0;i<RAW->getNLines();i++)
  {
	  TPrgLine* LINE = RAW->getLine(i);
    TString C = LINE->getToken(0);
		bool i0 = C.BeginsWith("LOOP");
    bool i1 = C.BeginsWith("ENDL");
		
    bool init = C.BeginsWith("INIT");
		
		if(init && nL>0) err(INITLOOPERR,LINE);
		if(init)
		{
		  nI++;
			if(nI>1) err(INITMANYERR,LINE);
		}
		
		if(nL==0 && !i0) // no loop, just continue
		{
		  if(!LINE->isExpanded())
			{
			  TPrgLine* LE = new TPrgLine(LINE->getCodeRaw());
				LE->setParent(LINE);
				EXP->addLine(LE);
				if(init) INITLINE = LE->getLineNumber();
			}
			else
			{
			  int n = LINE->getNExpanded();
				for(int j = 0;j<n;j++) 
				{
			    TPrgLine* LE = new TPrgLine(LINE->getExpanded(j)->getCodeRaw());
				  LE->setParent(LINE);
				  EXP->addLine(LE);
				}
			}
		}
		if(i1 && nL==0) err(LOOPNOTSTARTED,LINE);
    if(i0 && nL>0)  err(NESTEDLOOPS,LINE);
    if(i0) { loop1 = i; nL++; }
		if(i1)
		{
		  loop2 = i;
			nL--;
			int loopSize = loop2-loop1;
			TPrgLine* START = RAW->getLine(loop1);
			char *label = START->getLabel();
			
			int NT = START->getNTokens();
			if(NT!=2) err(LOOPERR,START);
			
			int TIMES = ATOI(START->getToken(1));
			
			if(TIMES<=0) err(LOOPSIZEERR,START);
			if(loopSize<=1) err(LOOPSIZEERR,START);
			
			for(int j = 0; j<TIMES; j++)
			{
			  for(int k = 1; k<loopSize; k++)
				{
				  TPrgLine* LINE = RAW->getLine(loop1+k);
		      if(!LINE->isExpanded())
			    {
			      TPrgLine* LE = new TPrgLine(LINE->getCodeRaw());
				    LE->setParent(LINE);
				    EXP->addLine(LE);
						if(j==0 && k==0) LE->setLabel(label);
			    }
			    else
			    {
			      int n = LINE->getNExpanded();
				    for(int h = 0;h<n;h++) 
				    {
			         TPrgLine* LE = new TPrgLine(LINE->getExpanded(h)->getCodeRaw());
				       LE->setParent(LINE);
				       EXP->addLine(LE);
						   if(j==0 && k==0 && h==0) LE->setLabel(label);
				    }
			    }
				}
			}	
		}
	}
	if(nI==0) err(INITNONEERR);
  if(nL>0) err(LOOPNOTCLOSED,RAW->getLine(loop1));
	//cout <<"INIT LINE IS = "<<INITLINE<<endl;
	return EXP;
}
/////////////////////////////////////////////////////////////////
TPrg* compact(TPrg* EXP)
{
  char line[400];
	char label[50];
	bool hasLabel = false;
	if(!EXP) return NULL;
	if(EXP->getNLines()==0) return NULL;
	
  TPrg* COMPACT = new TPrg();
	for(int i=0;i<EXP->getNLines();i++)
  {
	  TPrgLine* LINE = EXP->getLine(i);
		int ID = LINE->getId();
		
		switch(ID)
		{
		  case kDONTKNOW:
			break;
			case kCOMMENT:
			break;
			case kPRECOMP:
			break;
			case kCOMMAND:
			{
			  sprintf(line,"%s  ",LINE->getLabel());
				bool bit = false;
				for(int i = 0; i<LINE->getNTokens(); i++)
				{
				  char* token = LINE->getToken(i);
					if(!strcmp(token,"["))
					{
					  long int value = bitPattern(LINE, i);
						bit = true;
						sprintf(line,"%s %d",line,value);
					}
					if(!bit) 
					{
					  int x,y,z;
						if(isLabel(EXP,token,x))       sprintf(line,"%s %s",line,token);
						else if(isCommand(token,x,y,z))sprintf(line,"%s %s",line,token);
					  else if(isRegister(token,x,y)) sprintf(line,"%s %s",line,token);
						else if(isOperator(token,x))   sprintf(line,"%s %s",line,token);
						else sprintf(line,"%s %d",line,ATOI(token));
					}
					if(!strcmp(token,"]")) bit = false;					
				}				
			  TPrgLine* LE = new TPrgLine(line);
			  LE->setParent(LINE);
				COMPACT->addLine(LE);	
			}				
			break;
			case kSYMBOL:
			{
			  TString a = LINE->getLabel();
				if(a.Length()>0)
				{
				  //if the line has a label I have to add a NOP
					//statement in order to allow gotos to this
					//position in the code
					sprintf(line,"%s   NOP",LINE->getLabel());
			    TPrgLine* LE = new TPrgLine(line);
					LE->setParent(LINE);
					COMPACT->addLine(LE);					
				}
				sprintf(line,"Int_t %s;",LINE->getCode());
				//cout <<line<<endl;
        int e;
        gROOT->ProcessLineSync(line,&e);
        if(e) err(OPERERR,LINE);
			}
			break;
		}
	}
	return COMPACT;
}
/////////////////////////////////////////////////////////////////
long int bitPattern(TPrgLine* LINE, int go)
{
  long int bit = 0;
	
	int nT = LINE->getNTokens();
	int start = -1;
	int stop = -1;
	int nstart = 0;
	int nstop = 0;
	for(int i = go; i<nT; i++)
	{
	  char* token = LINE->getToken(i);
		if(!strcmp(token,"[")) { start = i; nstart++;}
		if(!strcmp(token,"]")) { stop = i; nstop++;}		
	}
	if(nstart!=1) err(BITPATTERR,LINE);
	if(nstop!=1)  err(BITPATTERR,LINE);
	if(stop<=start)  err(BITPATTERR,LINE);
	
	int N = stop-start;
	if(N<=1) err(BITPATTERR,LINE);
	
	
	for(int i = 1; i<N; i++)
	{
	  char* token = LINE->getToken(i+start);
		int V = ATOI(token);
	  if(V<=0) err(BITPATTERR,LINE);
		bit= bit | (int)pow(2.0,V-1);
	}
	
	return bit;
}
/////////////////////////////////////////////////////////////////
bool isLabel(TPrg* P, char* c,int& number)
{
  int n = P->getNLines();
	for(int i = 0; i<n; i++)
	{
	  TString a = P->getLine(i)->getLabel();
		if(a.Length()>0 && !strcmp(a.Data(),c))
		{
		  number = i;
			return true;
		}
	}
	return false;
}
/////////////////////////////////////////////////////////////////
bool isCommand(char* c,int& type, int& ntmin, int& ntmax)
{
  // NT is the number of parameters
	
	if(!strcmp(c,"NOP"))  { type = C_NOP;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"NAF"))  { type = C_NAF;  ntmin = 3; ntmax = 4; return true;}
  if(!strcmp(c,"NAFX")) { type = C_NAFX; ntmin = 3; ntmax = 4; return true;}
  if(!strcmp(c,"SPB"))  { type = C_SPB;  ntmin = 1; ntmax = 1; return true;}
  if(!strcmp(c,"BRU"))  { type = C_BRU;  ntmin = 1; ntmax = 1; return true;}
  if(!strcmp(c,"BRUR")) { type = C_BRUR; ntmin = 3; ntmax = 3; return true;}
  if(!strcmp(c,"SKIP")) { type = C_SKIP; ntmin = 3; ntmax = 3; return true;}
  if(!strcmp(c,"MOV"))  { type = C_MOV;  ntmin = 2; ntmax = 2; return true;}
  if(!strcmp(c,"MERG")) { type = C_MERG; ntmin = 1; ntmax = 2; return true;}
  if(!strcmp(c,"OUT"))  { type = C_OUT;  ntmin = 1; ntmax = 1; return true;}
  if(!strcmp(c,"PAR"))  { type = C_PAR;  ntmin = 1; ntmax = 1; return true;}
  if(!strcmp(c,"INC"))  { type = C_INC;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"DEC"))  { type = C_DEC;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"AND"))  { type = C_AND;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"OR"))   { type = C_OR;   ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"NOT"))  { type = C_NOT;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"RSH"))  { type = C_RSH;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"LSH"))  { type = C_LSH;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"MOD"))  { type = C_MOD;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"MUL"))  { type = C_MUL;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"DIV"))  { type = C_DIV;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"DLAY")) { type = C_DLAY; ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"SETC")) { type = C_SETC; ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"RTRN")) { type = C_RTRN; ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"END"))  { type = C_END;  ntmin = 0; ntmax = 0; return true;}
  if(!strcmp(c,"INIT")) { type = C_INIT; ntmin = 0; ntmax = 0; return true;}
	return false;
}
/////////////////////////////////////////////////////////////////
bool isOperator(char* c,int& type)
{
  if(!strcmp(c,"NONE")) { type = O_NONE; return true;}
  if(!strcmp(c,"ANY"))  { type = O_ANY;  return true;}
  if(!strcmp(c,"GT"))   { type = O_GT;   return true;}
  if(!strcmp(c,"LT"))   { type = O_LT;   return true;}
  if(!strcmp(c,"EQ"))   { type = O_EQ;   return true;}
  if(!strcmp(c,"NE"))   { type = O_NE;   return true;}
  if(!strcmp(c,"LE"))   { type = O_LE;   return true;}
  if(!strcmp(c,"GE"))   { type = O_GE;   return true;}
  return false;
}
/////////////////////////////////////////////////////////////////
bool isRegister(char* c,int& type,int& index)
{
  TString C=c;
  if(C.BeginsWith("PAT"))
  {
    type = R_PAT;
    if(!strcmp("PAT",c)) {index = 0; return true;}
    TString I = C(3,C.Length());
    index = ATOI(I.Data());
    if(index<0 || index>9) { index = 0; return false;}
    else return true;
  }
  if(C.BeginsWith("EX"))
  {
    type = R_EXX;
    if(!strcmp("EX",c)) {index = 0; return true;}
    TString I = C(2,C.Length());
    index = ATOI(I.Data());
    if(index<0 || index>9) { index = 0; return false;}
    else return true;
  }
  if(C.BeginsWith("CA"))
  {
    type = R_CA;
    if(!strcmp("CA",c)) {index = 0; return true;}
    TString I = C(2,C.Length());
    index = ATOI(I.Data());
    if(index<0 || index>9) { index = 0; return false;}
    else return true;
  }
  index = 0;
  return false;
}
/////////////////////////////////////////////////////////////////
int ATOI(char* v)
{
  TString N = v;
  if( N.BeginsWith("0X")  ||  N.EndsWith("H")) return strtol(v,NULL,16);
  if(!N.BeginsWith("0X")  &&  N.EndsWith("B")) return strtol(v,NULL,2);
  if( N.BeginsWith("0B")  && !N.EndsWith("H")) 
  {
    N=N(2,N.Length());
    return strtol(N.Data(),NULL,2);
  }
  if(N.IsDigit()) return atoi(v);
  TGlobal *g = (TGlobal*)gROOT->GetGlobal(N.Data(),kTRUE);
  if(!g) 
  {
    cout <<"Problem finding variable "<<N.Data()<<endl;
    err(VARIABLENOTDEF);
  }	
	else
	{
    int* value = (int*)g->GetAddress();
    if(!value) 
		{
      cout <<"Problem finding variable "<<N.Data()<<endl;
		  err(VARIABLENOTDEF);
	  }
		return (*value);
	}
	return -1;
}
/////////////////////////////////////////////////////////////////
int ATOI(const char* v)
{
  return ATOI((char*)v);
}
/////////////////////////////////////////////////////////////////
void compile(TPrg* PROG)
{
  char line[400];
	char label[50];
	if(!PROG) return;
	if(PROG->getNLines()==0) return;
	
	for(int i=0;i<PROG->getNLines();i++)
  {
	  TPrgLine* LINE = PROG->getLine(i);
		int nt = LINE->getNTokens()-1;
		int CMD, NTMIN, NTMAX;
		
		char* command = LINE->getToken(0);
		
		// check if command is valid
		if(!isCommand(command,CMD,NTMIN,NTMAX)) err(NOTVALIDCOMMAND,LINE);
		
		//check number of tokens
		if(nt<NTMIN || nt>NTMAX) err(INVALIDNUMBERCMD,LINE);
		
		switch(CMD)
		{
		  case  C_NOP:   makeNOP(LINE);	  break;
		  case  C_NAF:   makeNAF(LINE);	  break;
		  case  C_NAFX:  makeNAFX(LINE);  break;
		  case  C_SPB:   makeSPB(LINE);	  break;
		  case  C_BRU:   makeBRU(LINE);	  break;
		  case  C_BRUR:  makeBRUR(LINE);  break;
		  case  C_SKIP:  makeSKIP(LINE);  break;
		  case  C_MOV:   makeMOV(LINE);	  break;
		  case  C_MERG:  makeMERG(LINE);  break;
		  case  C_OUT:   makeOUT(LINE);	  break;
		  case  C_PAR:   makePAR(LINE);	  break;
		  case  C_INC:   makeINC(LINE);   break;
		  case  C_DEC:   makeDEC(LINE);	  break;
		  case  C_DLAY:  makeDLAY(LINE);  break;
		  case  C_SETC:  makeSETC(LINE);  break;
		  case  C_RTRN:  makeRTRN(LINE);	break;
		  case  C_END:   makeEND(LINE);	  break;
		  case  C_INIT:  makeINIT(LINE);	break;
		  case  C_AND:   makeAND(LINE);	  break;
		  case  C_OR:    makeOR(LINE);    break;
		  case  C_NOT:   makeNOT(LINE);   break;
		  case  C_RSH:   makeRSH(LINE);   break;
		  case  C_LSH:   makeLSH(LINE); 	break;
		  case  C_MOD:   makeMOD(LINE);   break;
		  case  C_MUL:   makeMUL(LINE);   break;
		  case  C_DIV:   makeDIV(LINE);   break;
		  case  C_ABORT: makeABORT(LINE); break;
										
		}		
  }
}

void  makeNOP(TPrgLine* LINE)
{
  LINE->setCompiledCode(C_NOP);
}
void  makeNAF(TPrgLine* LINE)
{
}
void  makeNAFX(TPrgLine* LINE)
{
}
void  makeSPB(TPrgLine* LINE)
{
}
void  makeBRU(TPrgLine* LINE)
{
}
void  makeBRUR(TPrgLine* LINE)
{
  LINE->setCompiledCode(C_BRUR);
}
void  makeSKIP(TPrgLine* LINE)
{
}
void  makeMOV(TPrgLine* LINE)
{
}
void  makeINC(TPrgLine* LINE)
{
}
void  makeDEC(TPrgLine* LINE)
{
}
void  makeAND(TPrgLine* LINE)
{
}
void  makeOR(TPrgLine* LINE)
{
}
void  makeNOT(TPrgLine* LINE)
{
}
void  makeRSH(TPrgLine* LINE)
{
}
void  makeLSH(TPrgLine* LINE)
{
}
void  makeMOD(TPrgLine* LINE)
{
}
void  makeMUL(TPrgLine* LINE)
{
}
void  makeDIV(TPrgLine* LINE)
{
}
void  makeMERG(TPrgLine* LINE)
{
}
void  makeOUT(TPrgLine* LINE)
{
}
void  makePAR(TPrgLine* LINE)
{
}
void  makeDLAY(TPrgLine* LINE)
{
}
void  makeSETC(TPrgLine* LINE)
{
}
void  makeCNAF(TPrgLine* LINE)
{
}
void  makeIF(TPrgLine* LINE)
{
}
void  makeROUT(TPrgLine* LINE)
{
}
void  makeCROUT(TPrgLine* LINE)
{
}
void  makeABORT(TPrgLine* LINE)
{
  LINE->setCompiledCode(C_ABORT);
}
void  makeRTRN(TPrgLine* LINE)
{
  LINE->setCompiledCode(C_RTRN);
}
void  makeEND(TPrgLine* LINE)
{
  LINE->setCompiledCode(C_END);
}
void  makeINIT(TPrgLine* LINE)
{
  LINE->setCompiledCode(C_INIT);
}











